
/*!
 * Simple quick and dirty wrapper functions for the CMU1394 library
 * 
 * Copyright (c) 2008 by <Philip Birch/University of Sussex>
 * Version 0.1
 */
#include "stdafx.h"
#include "CMUCameraGrab.h"


/*!
 * \brief
 * CMU camera class
 */
C1394Camera Camera;

long numFrames=0;
clock_t clocking=0;
/*!
 * \brief
 * Converts frames per second into DCAM frame rate code
 * 
 * \param fps
 * input desired fps.
 * 
 * \returns
 * DCAM frame rate code or -1 if input parameter is not supported
 * 
 
 * This function converts a real floating point frames per second 
 * into the correct DCAM frame rate code. The supported rates are
 * 1.875,3.75,7.5,15,30,60,120,240 frames per second. The function
 * uses rounding in converting floats to integers to take into account
 * machine errors. Note it does not test to see if the camera supports the mode.
 * 
 */
int getFrameRateCode(float fps)
{
	int fr;
	int ifps=(int)(fps+0.5f);
	switch (ifps)
	{
	case 2:
		fr=0;
		break;
	case 4:
		fr=1;
		break;
	case 8:
		fr=2;
		break;
	case 15:
		fr=3;
		break;
	case 30:
		fr=4;
		break;
	case 60:
		fr=5;
		break;
	case 120:
		fr=6;
		break;
	case 240:
		fr=7;
		break;
	default:
		printf("Unsupported DCAM standard frame rate.\n");
		fr=-1;
		break;
	}
	return fr;



	}
/*!
 * \brief
 * Converts DCAM frame rate code into frames per second.
 * 
 * \param frameRateCode
 * DCAM frame rate code 0-7.
 * 
 * \returns
 * fps or -1 if frame rate code is not between 0 and 7
 * 
 */
float getFrameRate(int frameRateCode)
{
	float f=0;
	switch (frameRateCode)
	{
	case 0:
		f=1.875;
		break;
	case 1:
		f=3.75;
		break;
	case 2:
		f=7.5;
		break;
	case 3:
		f=15;
		break;
	case 4:
		f=30;
		break;
	case 5:
		f=60;
		break;
	case 6:
		f=120;
		break;
	case 7:
		f=240;
		break;
	default:
		printf("Unknown frame rate code\n");
		f=-1;
		break;
	}
	return f;
}

///set up camera given width, height, fps and colour space
/*!
 * \brief
 * Sets up the camera
 * 
 * \param width
 * Video width.
 * 
 * \param height
 * Video height
 * 
 * \param fps
 * Frames per second.
 * 
 * \param col
 * Colour space
 * 
 * \returns
 * 0 if okay, or else -1
 * 
 */
int setupFWCamera(int width, int height, float fps, enum ColourSpace col)
{
	int fr=0,md=0;
	int res=getModeFormat(&fr,&md,width,height,&col);
	if(res==-1)
	{
		printf("Error:Resolution not defined by DCAM standard\n");
		return -1;
	}
	else if(res==-2)
	{
		printf("Warning: Unsupport colour mode selected, defaulting to RGB\n");
	}
	int frc=getFrameRateCode(fps);
	if(frc==-1)
		return -1;

	return setupFWCamera(fr,md,frc);
}
///set up camera given a DCAM format, mode, and frame rate code
int setupFWCamera(int format, int mode, int frameRate)
{
		if(Camera.CheckLink() != CAM_SUCCESS)
        {
                printf("\nNo Link.");
                return -1;
        }
		if(Camera.InitCamera() != CAM_SUCCESS)
        {
                printf("\nInitialization failed.");
                return -1;
        }
        if(Camera.SetVideoFormat(format) != CAM_SUCCESS)
        {
                printf("\nCould not set video format.");
                return -1;
        }
        if(Camera.SetVideoMode(mode) != CAM_SUCCESS)
        {
                printf("\nCould not set video mode.");
                return -1;
        }
        if(Camera.SetVideoFrameRate(frameRate) != CAM_SUCCESS)
        {
                printf("\nCould not set frame rate.");
                return -1;
		}
		return 0;
}
///sets up camera with 640x480 RGB and 15fps
int setupFWCamera(void)
{

	return setupFWCamera(0,5,4);	
}
/// \brief starts the camera acquisition process
/// \returns 0 if okay or else -1
int startImageAcquisition()
{
	if(Camera.StartImageAcquisition() != CAM_SUCCESS)
        {
                printf("\nCould not start image acquisation.");
                return -1;
        }
	return 0;
}

/*!
 * \brief
 * Returns the real fps that is captured
 * 
 * \returns
 * fps
 * 
 */
float getRealFps()
{
	clock_t gap=clock()-clocking;
	clocking=clock();
	float fps=numFrames/(float)gap*CLOCKS_PER_SEC;
	numFrames=0;
	return fps;
}



///gets an image off the acquisition buffer. NB IplImage img is always unsigned 8 bit 3 channel RGB no matter what the camera settings are
int getFWImage(IplImage *img)
{
	 if(Camera.AcquireImage() != CAM_SUCCESS)
        {       
                printf("\nCould not acquire image.");
                return -1;
        }
		numFrames++;
		Camera.getRGB((unsigned char*)(img->imageData), img->imageSize);
	return 0;     
}
/// \brief stops the acquisition
/// \returns
/// 0
int stopFWCamera()
{
	Camera.StopImageCapture();
	return 0;
}


/*!
 * \brief
 * Converts colour space enum into human readable string
 * 
 * \param col
 *  Colour space
 * 
 * \param name
 * Pre-allocated char array, must be at least 7 bytes long
 * 
 * \returns
 * 0
 * 
 * Helper function
 */
int getColourString(enum ColourSpace col,char *name)
{
	switch(col)
	{
	case YUV444:
		strcpy(name,"YUV444");
		break;
	case YUV422:
		strcpy(name,"YUV422");
		break;
	case YUV411:
		strcpy(name,"YUV411");
		break;
	case RGB:
		strcpy(name,"RGB");
		break;
	case MONO8:
		strcpy(name,"MONO8");
		break;
	case MONO16:
		strcpy(name,"MONO16");
		break;
	default:
		return -1;
	}
	return 0;
}

/*!
 * \brief
 * Get the width, height and colour space for a given mode and format
 * 
 * \param format
 *  DCAM format.
 * 
 * \param mode
 * DCAM mode.
 * 
 * \param width
 * Outputs the width.
 * 
 * \param height
 * Outputs the height.
 * 
 * \param col
 * Outputs the colour space.
 * 
 * \returns
 * -1 on error, 0 otherwise
 * 
 * 
 * Helper function
 */
int getWidthHeight(int format, int mode, int *width,int *height, enum ColourSpace *col)
{
	switch (format)
	{
	case 0:
		{
			switch (mode)
			{
			case 0:
				*width=160;
				*height=120;
				*col=YUV444;
				break;
			case 1:
				*width=320;
				*height=240;
				*col=YUV422;
				break;
			case 2:
				*width=640;
				*height=480;
				*col=YUV411;
				break;
			case 3:
				*width=640;
				*height=480;
				*col=YUV422;
				break;
			case 4:
				*width=640;
				*height=480;
				*col=RGB;
				break;
			case 5:
				*width=640;
				*height=480;
				*col=MONO8;
				break;
			case 6:
				*width=640;
				*height=480;
				*col=MONO16;
				break;
			default:
				printf("Unsupported mode\n");
				return -1;

			}
			break;
		}//case 0
	case 1:
		{
			switch(mode)
			{
			case 0:
				*width=800;
				*height=600;
				*col=YUV422;
				break;
			case 1:
				*width=800;
				*height=600;
				*col=RGB;
				break;
			case 2:
				*width=800;
				*height=600;
				*col=MONO8;
				break;
			case 3:
				*width=1024;
				*height=768;
				*col=YUV422;
				break;
			case 4:
				*width=1024;
				*height=768;
				*col=RGB;
				break;
			case 5:
				*width=1024;
				*height=768;
				*col=MONO8;
				break;
			case 6:
				*width=800;
				*height=600;
				*col=MONO16;
				break;
			case 7:
				*width=1024;
				*height=768;
				*col=MONO16;
				break;
			default:
				printf("Unsupported mode\n");
				return -1;
			}
			break;
		}//case 1
	case 2:
		{
			switch(mode)
			{
			case 0:
				*width=1280;
				*height=960;
				*col=YUV422;
				break;
			case 1:
				*width=1280;
				*height=960;
				*col=RGB;
				break;
			case 2:
				*width=1280;
				*height=960;
				*col=MONO8;
				break;
			case 3:
				*width=1600;
				*height=1200;
				*col=YUV422;
				break;
			case 4:
				*width=1600;
				*height=1200;
				*col=RGB;
				break;
			case 5:
				*width=1600;
				*height=1200;
				*col=MONO8;
				break;
			case 6:
				*width=1280;
				*height=960;
				*col=MONO16;
				break;
			case 7:
				*width=1600;
				*height=1200;
				*col=MONO16;
				break;
			default:
				printf("Unsupported mode\n");
				return -1;
			}
			break;
		}//case 2
	default:
		printf("Unsupported format\n");
		return -1;
	}//end format switch
	return 0;
}



/*!
 * \brief
 * Gets the mode and format given a video width, height and colour space
 * 
 * \param format
 * DCAM format to be returned
 * 
 * \param mode
 * DCAM mode to be returned
 * 
 * \param width
 * input desired video width
 * 
 * \param height
 * input desired video height
 * 
 * \param col
 * input desired video colour model
 * 
 * \returns
 * -1 if width and height are not DCAM supported, 
 * -2 if the colour space is not supported for the selected width and height. In this case mode, format and col are 
set to RGB.
0 if sucessfull
 * 
 * This is a help function to find a mode and format for a given width, height and colour space. It does not
 * test to see if the camera supports the mode.
 */
int getModeFormat(int *format, int *mode, int width, int height,enum ColourSpace *col )
{
	if((width==160)&(height==120)|(width==320)&(height==240)|(width==640)&(height==480))
	{
		*format=0;
	}
	else if((width==800)&(height==600)|(width==1024)&(height==768))
	{
		*format=1;
	}
	else if((width==1280)&(height==960)|(width==1600)&(height==1200))
	{
		*format=2;
	}
	else
	{
		*format=-1;
		*mode=-1;
		return -1;
	}

	switch(*format)
	{
	case 0:
		{
			if(width==160)
			{
				*mode=0;
				if(*col!=YUV444)
				{	
					*col=YUV444;
					return -2;
				}
				return 0;
			}
			else if(width==320)
			{
				*mode=1;
				if(*col!=YUV422)
				{
					*col=YUV422;
					return -2;	
				}
				return 0;
			}
			else if(width==640)
			{
				switch(*col)
				{
				case YUV411:
					*mode=2;
					return 0;
					break;
				case YUV422:
					*mode=3;
					return 0;
					break;
				case RGB:
					*mode=4;
					return 0;
					break;
				case MONO8:
					*mode=5;
					return 0;
					break;
				case MONO16:
					*mode=6;
					return 0;
					break;
				default:
					*col=RGB;
					*mode=4;
					return -2;
				}
			}


		}
	case 1:
		{
			if(width==800)
			{
				switch(*col)
				{
				case YUV422:
					*mode=0;
					return 0;
					break;
				case RGB:
					*mode=1;
					return 0;
					break;
				case MONO8:
					*mode=2;
					return 0;
					break;
				case MONO16:
					*mode=6;
					return 0;
					break;
				default:
					*mode=1;
					*col=RGB;
					return -2;
					break;
				}
			}
			else if(width==1024)
			{
				switch(*col)
				{
				case YUV422:
					*mode=3;
					return 0;
					break;
				case RGB:
					*mode=4;
					return 0;
					break;
				case MONO8:
					*mode=5;
					return 0;
					break;
				case MONO16:
					*mode=7;
					return 0;
					break;
				default:
					*mode=4;
					*col=RGB;
					return -2;
				}
			}

		}//end case
	case 2:
		{
			if(width==1280)
			{
				switch (*col)
				{
				case YUV422:
					*mode=0;
					return 0;
					break;
				case RGB:
					*mode=1;
					return 0;
					break;
				case MONO8:
					*mode=2;
					return 0;
					break;
				case MONO16:
					*mode=6;
					return 0;
					break;
				default:
					*mode=1;
					*col=RGB;
					return -2;
					break;
				

				}
			}
			else if(width==1600)
			{
				switch (*col)
				{
				case YUV422:
					*mode=3;
					return 0;
					break;
				case RGB:
					*mode=4;
					return 0;
					break;
				case MONO8:
					*mode=5;
					return 0;
					break;
				case MONO16:
					*mode=7;
					return 0;
					break;
				default:
					*mode=4;
					*col=RGB;
					return -2;
					break;

				}
			}
		}//end case

	}
	return 0;

}


/*!
 * \brief
 * Convert a colour BRG image to RGB
 * 
 * \param img
 * IplImage to covert colour order. This must be 8 bit unsigned and 3 channel. No checking is done for speed. 
 * 
 * Convert a colour BRG image to RGB. Colour from the camera is in BRG, even though Microsoft lie call it RGB. They do that a lot.
 * 
 * 
 */
void BRG2RGB(IplImage *img)
{
	uchar tmp;
	for(int j=0;j<img->height;j++)
		for(int i=0;i<img->width;i++)
		{
			tmp=RPIX8U(img,i,j);
			RPIX8U(img,i,j)=BPIX8U(img,i,j);
			BPIX8U(img,i,j)=tmp;
		}
}


/*!
 * \brief
 * Returns the current camera format and mode.
 * 
 * \param format
 * output format.
 * 
 * \param mode
 * output mode.
 * 
 * \returns
 * 0 if okay, -1 otherwise
 */
int getCurrentCameraFormat(int *format, int *mode)
{
	*format=Camera.GetVideoFormat();
	if(*format==-1)
		return -1;
	*mode=Camera.GetVideoMode();
	if(*mode==-1)
		return -1;
	return 0;
}
/*!
 * \brief
 * Prints supported modes to the console.
 * 
 * \returns
 * 0
 * 
 */
int listSupportedModes()
{
	for(int format=0;format<3;format++)
		for(int mode=0;mode<7;mode++)
		{
			if(Camera.HasVideoMode(format,mode)==TRUE)
			{
				printf("Format %d Mode %d ",format,mode);
				int wd=0,ht=0;
				ColourSpace col;
				getWidthHeight(format,mode,&wd,&ht,&col);
				char str[8];
				getColourString(col,str);
				printf("Width %d Height %d Colour Space %s supported by camera\nAt frame rates:\n",wd,ht,str);
				for(int fr=0;fr<8;fr++)
				{
					if(Camera.HasVideoFrameRate(format,mode,fr)==TRUE)
					{
						printf("Code %d, rate %.3f ",fr,getFrameRate(fr));
					}
					
				}
				printf("\n");
			}
		}
	return 0;
}